package be.iminds.ilabt.jfed.espec.bundle;

import be.iminds.ilabt.jfed.espec.model.ExperimentSpecification;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * Bundle a collections of files, which contains the ExperimentSpecification description yaml file
 */
public interface ESpecBundle {
    public static class ESpecBundleInitException extends Exception {
        public ESpecBundleInitException() { }
        public ESpecBundleInitException(String s) { super(s); }
        public ESpecBundleInitException(String s, Throwable throwable) { super(s, throwable); }
        public ESpecBundleInitException(Throwable throwable) { super(throwable); }
        public ESpecBundleInitException(String s, Throwable throwable, boolean b, boolean b1) { super(s, throwable, b, b1); }
    }

    static String ESPEC_DESCRIPTION_FILENAME = "experiment-specification.yml";

    /**
     * Gets the files in the root if this bundle (non recursive). This includes any directories in the root (but not their content).
     * Dirs returned may or may not end with a /
     *
     * @return
     */
    @Nonnull List<String> getFileNames();

    /**
     * Get the byte content of a file.
     *
     * @param fileName the full (relative) path of the file in this bundle
     * @return
     * @throws IOException
     * @throws IllegalArgumentException If a dir is given instead of a file.
     * @throws java.io.FileNotFoundException If the fileName does not specify a file in the bundle.
     */
    @Nonnull byte[] getFileContent(@Nonnull String fileName) throws IOException;

    /**
     *
     * @param fileName  the full (relative) path of the file/dir in this bundle. fileName may not end with /, both are guaranteed to work.
     * @return
     * @throws IOException
     */
    boolean isDir(@Nonnull String fileName) throws IOException;

    /**
     * Get the files in each dir, non recursive. This can return subdirs, so make sure you check isDir on each file!
     * Dirs returned may or may not end with a /
     *
     * @param dirFileName the full (relative) path of the dir in this bundle. dirFileName may not end with /, both are guaranteed to work.
     * @return a list with the full (relative) names of the files and dirs in the requested dir in this bundle
     * @throws IOException
     */
    List<String> getDirFiles(@Nonnull String dirFileName) throws IOException;

    default boolean isExperimentSpecificationPresent() {
        return getFileNames().contains(ESPEC_DESCRIPTION_FILENAME);
    }

    @Nonnull
    default String getExperimentSpecificationYml() throws IOException {
        assert getFileNames().contains(ESPEC_DESCRIPTION_FILENAME);
        return new String(getFileContent(ESPEC_DESCRIPTION_FILENAME), StandardCharsets.UTF_8);
    }

    @Nonnull
    ExperimentSpecification getExperimentSpecification();
}
